/*++

Copyright (c) 2004 Alexander A. Telyatnikov (Alter)

Module Name:
    PciIds.cpp

Abstract:
    Parsing module for well-known PCI ID list by Martin Mares

Author:
    Alexander A. Telyatnikov (Alter)

Environment:
    User mode only

Notes:

    THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
    OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
    NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
    THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Revision History:

--*/

//#ifdef _AFXDLL_
//#include "stdafx.h"
//#endif
#include <windows.h>
#include <stdio.h>
#include "PciIds.h"

#define VendorList_filename   "pci.ids"
#define VendorList_filename2  "vendors.txt"


str2id _v_names[] = {
    {0x0000, -1, "Gammagraphx, Inc."},
    {0x0033, -1, "Paradyne Corp."},
    {0x003D, -1, "Lockheed Martin Corp."},
    {0x0070, -1, "Hauppauge Computer Works Inc."},
    {0x0675, -1, "Dynalink"},
    {0x0700, -1, "Lava Computer MFG Inc."},
    {0x0815, -1, "LinTech GmbH"},
    {0x0871, -1, "Berkom"},
    {0x0914, -1, "Shanghai Dare Technologies Ltd."},
    {0x09C1, -1, "Arris"},
    {0x0A89, -1, "BREA Technologies Inc."},
    {0x0E11, -1, "Compaq Computer Corp."},
    {0x1000, -1, "LSI Logic"},
    {0x1002, -1, "ATI Technologies"},
    {0x1003, -1, "ULSI"},
    {0x1004, -1, "VLSI Technology"},
/*        {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},
    {0x1002, "ATI"},*/
    {0x1022, -1, "AMD"},
    {0x102B, -1, "Matrox"},
    {0x1039, -1, "SiS"},
    {0x103C, -1, "Hewlett-Packard Company"},
    {0x105a, -1, "Promise"},
    {0x1065, -1, "Texas Microsystems"},
    {0x1078, -1, "Cyrix"},
    {0x1080, -1, "Cypress"},
    {0x1095, -1, "CMD"},
    {0x10b9, -1, "Acer Labs"},
    {0x10de, -1, "nVIDIA"},
    {0x10ec, -1, "ATI"},
    {0x1103, -1, "High Point"},
    {0x1106, -1, "VIA"},
    {0x1166, -1, "Server Works"},
    {0x16ca, -1, "Cenatek"},
    {0x8086, -1, "Intel"},
    {0xffff, 0, 0}
};

str2id _d_names[] = {
    {0x808624cb, -1, "ICH4 ATA 100"},
    {0x10b95229, -1, "Aladdin ATA/ATA33 controller"},
    {0x10de01bc, -1, "nForce ATA"},
    {0x10227441, -1, "AMD 768 ATA"},
    {0x11060571, -1, "ATAxxx controller"},
    {0x10395513, -1, "5591 ATA33"},
    {0x10950649, -1, "CMD 649 ATA 100"},
    {0x1080c693, -1, "82C693 ATA"},
    {0x10780102, -1, "5530 ATA33"},
    {0x11660211, -1, "ROSB4 ATA33"},
    {0x105a4d68, -1, "TX2 ATA100"},
    {0x11030004, -1, "HPT366/368/370/372 ATA"},
    {0x16ca0001, -1, "Rocket Drive"},
    {0x10025046, -1, "Rage 128 Pro"},
    {0xffff, 0, 0}
};

str2id _c_names[] = {
    {0x01, -1, "Mass Storage"},
    {0x02, -1, "Network Card"},
    {0x03, -1, "Video Adapter"},
    {0x04, -1, "Multimedia Device"},
    {0x05, -1, "Memory"},
    {0x06, -1, "Bridge Device"},
    {0x07, -1, "Simple Communication Device"},
    {0x08, -1, "Base Peripheral"},
    {0x09, -1, "Input Device"},
    {0x0a, -1, "Docking Device"},
    {0x0b, -1, "Processor"},
    {0x0c, -1, "Serial Bus"},
    {0x0d, -1, "Wireless"},
    {0x0e, -1, "Intelligent I/O"},
    {0x0f, -1, "Satellite Communication"},
    {0x10, -1, "En/Decryption"},
    {0x11, -1, "Data Aquisition & Signal Processing"},
    {0xffff, 0, 0},
};

str2id _s_names[] = {
    {0xffff, 0, 0},
};

str2id* v_names = &_v_names[0];
str2id* d_names = &_d_names[0];
str2id* c_names = &_c_names[0];
str2id* s_names = &_s_names[0];
char* ven_buff;

ULONG
parse_hex(char* txt)
{
    char a;
    ULONG v = 0;
    ULONG d;

    while(a = *txt) {
        if(a >= '0' && a <= '9') {
            d = a-'0';
        } else
        if(a >= 'a' && a <= 'f') {
            d = a-'a'+10;
        } else
        if(a >= 'A' && a <= 'F') {
            d = a-'A'+10;
        } else {
            break;
        }
        v = (v << 4) + d;
        txt++;
    }
    return v;
} // end parse_hex()

void
load_pci_info()
{
    HANDLE h;
    ULONG len;
    ULONG readlen;
    ULONG i, p;
//    ULONG j;
    ULONG strcount;
    char a;
    BOOLEAN comment = FALSE;
    BOOLEAN datastr = FALSE;
    ULONG v_count = 0;
    ULONG d_count = 0;
    ULONG last_v = 0xffffffff;
    BOOLEAN expect_hex;
    BOOLEAN expect_str;
    char* hex_start;
    ULONG tab_count;

    h = CreateFile(VendorList_filename, GENERIC_READ,
                   FILE_SHARE_READ | FILE_SHARE_WRITE,
                   NULL,
                   OPEN_EXISTING,
                   FILE_ATTRIBUTE_NORMAL/* | FILE_FLAG_NO_BUFFERING*/,  NULL);
    if(!h || h == ((HANDLE)(-1)) ) {
        h = CreateFile(VendorList_filename2, GENERIC_READ,
                       FILE_SHARE_READ | FILE_SHARE_WRITE,
                       NULL,
                       OPEN_EXISTING,
                       FILE_ATTRIBUTE_NORMAL/* | FILE_FLAG_NO_BUFFERING*/,  NULL);
        if(!h || h == ((HANDLE)(-1)) ) {
            return;
        }
    }
    len = GetFileSize(h, NULL);
    if(!len) {
        CloseHandle(h);
        return;
    }
    ven_buff = (char*)GlobalAlloc(GMEM_DISCARDABLE, len+1);
    if(!ven_buff) {
        CloseHandle(h);
        return;
    }
    if(!ReadFile(h, ven_buff, len, &readlen, NULL)) {
        GlobalFree(ven_buff);
        CloseHandle(h);
        return;
    }
    if(len != readlen) {
        GlobalFree(ven_buff);
        CloseHandle(h);
        return;
    }
    ven_buff[len] = 0;
    strcount = 0;
    tab_count = 0;
    for(i=0; i<len; i++) {
        a = ven_buff[i];
        switch(a) {
        case '\n':
new_line:
            strcount++;
            comment = FALSE;
            datastr = FALSE;
            tab_count = 0;
            break;
        case '\r':
            break;
        case ' ':
        case '\t':
            if(comment || datastr) {
                break;
            }
            datastr = FALSE;
            for(;i<len; i++) {
                a = ven_buff[i];
                switch(a) {
                case ' ':
                case '\t':
                    break;
                case '\n':
                case '\r':
                    goto new_line;
                default:
                    if(!datastr) {
                        d_count++;
                        datastr = TRUE;
                    }
                }
            }
        case ';':
        case '#':
            comment = TRUE;
            break;
        default:
            if(comment || datastr) {
                break;
            }
            if(a == 'C') {
                i = len - 1;
                break;
            }
            v_count++;
            datastr = TRUE;
        }
    }

    v_names = (str2id*)GlobalAlloc(GMEM_DISCARDABLE, (v_count+1)*sizeof(str2id));
    if(!v_names) {
        v_names = &_v_names[0];
        GlobalFree(ven_buff);
        CloseHandle(h);
        return;
    }
    d_names = (str2id*)GlobalAlloc(GMEM_DISCARDABLE, (d_count+1)*sizeof(str2id));
    if(!d_names) {
        d_names = &_d_names[0];
        GlobalFree(v_names);
        v_names = &_v_names[0];
        GlobalFree(ven_buff);
        CloseHandle(h);
        return;
    }

    comment = FALSE;
    datastr = FALSE;
    v_count = 0;
    d_count = 0;
    tab_count = 0;
    p = 0;
    for(i=0; i<len; i++) {
        if(i && i*10/len != p) {
            printf(".");
            p = i*10/len;
        }
        a = ven_buff[i];
        switch(a) {
        case '\n':
new_line_2:
            strcount++;
            comment = FALSE;
            datastr = FALSE;
            tab_count = 0;
            break;
        case '\r':
            break;
        case ' ':
        case '\t':
            if(comment || datastr) {
                break;
            }
            if(last_v == 0xffffffff) {
                comment = TRUE;
                break;
            }
            tab_count++;
            expect_hex = TRUE;
            expect_str = FALSE;
            hex_start = NULL;
            for(i++;i<len; i++) {
                a = ven_buff[i];
                switch(a) {
                case ' ':
                case '\t':
                    if(expect_hex && hex_start) {
                        //sscanf(hex_start, "%x", &d_names[d_count].id);
                        d_names[d_count].id = parse_hex(hex_start);
                        d_names[d_count].id |= (last_v << 16);
                        d_names[d_count].str = "";
                        datastr = TRUE;
                        expect_hex = FALSE;
                        expect_str = TRUE;
                        hex_start = NULL;
                    } else {
                        tab_count++;
                    }
                    break;
                case '\n':
                case '\r':
                    if(expect_str && hex_start) {
                        d_names[d_count].str = hex_start;
                        ven_buff[i] = 0;
                    }
                    if(datastr) {
                        d_count++;
                    }
                    goto new_line_2;
                default:
                    if(expect_hex) {
                        if(!hex_start) {
                            hex_start = ven_buff+i;
                        }
                    } else
                    if(expect_str) {
                        if(!hex_start) {
                            hex_start = ven_buff+i;
                        }
                    }
                }
            }
            if(datastr) {
                d_count++;
            }
        case ';':
        case '#':
            comment = TRUE;
            break;
        default:
            if(comment || datastr) {
                break;
            }
            if(a == 'C') {
                i = len - 1;
                break;
            }
            expect_hex = TRUE;
            expect_str = FALSE;
            hex_start = NULL;
            for(;i<len; i++) {
                a = ven_buff[i];
                switch(a) {
                case ' ':
                case '\t':
                    if(expect_hex && hex_start) {
                        //sscanf(hex_start, "%x", &last_v);
                        last_v = parse_hex(hex_start);
                        v_names[v_count].id = last_v;
                        v_names[v_count].str = "";
                        datastr = TRUE;
                        expect_hex = FALSE;
                        expect_str = TRUE;
                        hex_start = NULL;
                    } else {
                        tab_count++;
                    }
                    break;
                case '\n':
                case '\r':
                    if(expect_str && hex_start) {
                        v_names[v_count].str = hex_start;
                        ven_buff[i] = 0;
                    }
                    if(datastr) {
                        v_count++;
                    }
                    goto new_line_2;
                default:
                    if(expect_hex) {
                        if(!hex_start) {
                            hex_start = ven_buff+i;
                        }
                    } else
                    if(expect_str) {
                        if(!hex_start) {
                            hex_start = ven_buff+i;
                        }
                    }
                }
            }
            if(datastr) {
                v_count++;
            }
        }
    }
    printf(".\n");
    v_names[v_count].str = NULL;
    d_names[d_count].str = NULL;
} // end load_pci_info()
